{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
   ],
   "source": [
    "import random\n",
    "import string\n",
    "import copy\n",
    "\n",
    "class Building():\n",
    "    def __init__(self, num_of_floor, num_of_passenger):\n",
    "        self.num_of_floor = num_of_floor\n",
    "        self.num_of_passenger = num_of_passenger\n",
    "        self.passengers = [] #list of all passengers\n",
    "        self.random_passengers()   # generate random passengers\n",
    "\n",
    "    def random_passengers(self):\n",
    "        for i in range(self.num_of_passenger):\n",
    "            p = Passenger(random.randrange(self.num_of_floor), random.randrange(self.num_of_floor)) #generates an origin and a destination\n",
    "\n",
    "            # make sure the origin and destination is different\n",
    "            while p.origin == p.destination:\n",
    "                p.destination = random.randrange(self.num_of_floor)\n",
    "\n",
    "            self.passengers.append(p) #add random passenger to building\n",
    "\n",
    "###########################################################################\n",
    "\n",
    "class Passenger():\n",
    "    def __init__(self, origin, destination):\n",
    "        self.name = \"\".join(random.choice(string.ascii_uppercase)) +\\\n",
    "                    \"\".join(random.choice(string.ascii_lowercase) for _ in range(random.randrange(2, 8)))\n",
    "                    # a random name for the passenger for console display\n",
    "        self.origin = origin\n",
    "        self.destination = destination\n",
    "        self.deliver = 0   # marker of if the passenger is delivered, 0=undelivered 1=delivered\n",
    "        self.cost = 0   # the cost to deliver the passenger\n",
    "\n",
    "    def compute_cost(self, movement):\n",
    "        self.cost = movement - abs(self.origin - self.destination)\n",
    "        print(\"Drop off\", self.name, \"at floor\", self.destination, \"with cost\", self.cost)\n",
    "\n",
    "###########################################################################\n",
    "\n",
    "class Elevator():\n",
    "    def __init__(self, capacity, position=0, direction=1):\n",
    "        self.capacity = capacity   # max allowed people in the elevator\n",
    "        self.position = position   # current position of the elevator, defalut to 0\n",
    "        self.movement = 0   # count the total movement of the elevator, used to calculate cost function\n",
    "        self.direction = direction   # last moving direction of the elevator, 0=down 1=up, default to 1\n",
    "        self.carrying = []   # a collection of passengers currently inside the elevator\n",
    "\n",
    "    def move(self):\n",
    "        # move one floor following the given direction\n",
    "        if self.direction == 1:\n",
    "            self.position += 1\n",
    "        else:\n",
    "            self.position -= 1\n",
    "\n",
    "        self.movement += 1   # increment the total movement by 1 as the elevator has moved one more step\n",
    "\n",
    "    def drop_passenger(self):\n",
    "        # go through all passengers on the elevator to check if they reached their destination\n",
    "        dropped = []\n",
    "        for key, object in enumerate(self.carrying):\n",
    "            if object.destination == self.position: #mark if passenger is at destination floor\n",
    "                dropped.append(key) #add passenger to be dropped\n",
    "                object.compute_cost(self.movement)\n",
    "        \n",
    "        if dropped: #removed dropped passengers from elevator\n",
    "            self.carrying = [p for ind, p in enumerate(self.carrying) if ind not in dropped]\n",
    "\n",
    "\n",
    "    def reverse(self):\n",
    "        if self.direction == 1:\n",
    "            self.direction = 0\n",
    "        else:\n",
    "            self.direction = 1\n",
    "\n",
    "###########################################################################\n",
    "\n",
    "\n",
    "# Strategy 1 has 3 principles:\n",
    "# 1. The elevator will always favour its current moving direction.\n",
    "# 2. Unless reached its capacity, the elevator will take all encountered passengers that are going in the same direction.\n",
    "# 3. The elevator will reverse its current moving direction if:\n",
    "#    1. There is no passenger currently inside the elevator.\n",
    "#    2. There is no undelivered passenger in the elevator's current direction\n",
    "\n",
    "def strategy_1(b, e): #inputs a building and an elevator\n",
    "    awaiting = copy.copy(b.passengers) #list of passengers in the building \n",
    "    e.carrying = [] #reset elevator\n",
    "    while len(awaiting) > 0 or len(e.carrying) > 0: #while there are people to be picked up or people to deliver\n",
    "        print(\"The elevator is at\", e.position, \"carrying\", len(e.carrying), \"passengers\")\n",
    "\n",
    "        # at each floor, first check if there is a passenger to drop\n",
    "        e.drop_passenger()\n",
    "\n",
    "        # if the elevator still has capacity, check if there is passenger to carry\n",
    "        if len(awaiting) > 0 and len(e.carrying) < e.capacity:\n",
    "            DEL = []\n",
    "            for key, object in enumerate(awaiting): #loop through all waiting passengers in the building\n",
    "                if e.position == object.origin:\n",
    "                    if ((object.destination - object.origin) > 0) == e.direction:   # if the elevator is going in the direction the passenger wants to go\n",
    "                        e.carrying.append(object) #pick up passenger\n",
    "                        DEL.append(key) # add passenger to be removed from elevator calls\n",
    "                        print(\"Picking up\", object.name, \"at floor\", e.position)\n",
    "                        if len(e.carrying) == e.capacity:\n",
    "                            break   # when capacity is reached, stop checking if there are passengers to pick up (break for loop)\n",
    "            if DEL:\n",
    "                awaiting = [p for ind, p in enumerate(awaiting) if ind not in DEL] #remove dropped passengers from awaiting list\n",
    "\n",
    "        # decide if to reverse\n",
    "        if len(e.carrying) == 0: #if the elevator isn't carrying any passengers\n",
    "            reverse = 1 #reverse direction unless\n",
    "            for object in awaiting: #there are passengers to pick up in the current direction\n",
    "                if ((object.origin - e.position) > 0) == e.direction:\n",
    "                    reverse = 0\n",
    "                    break\n",
    "            if reverse == 1:\n",
    "                e.reverse()\n",
    "\n",
    "        # if there's a passenger waiting in the same direction, move one floor; if not, keep the position and re-evaluate\n",
    "        if len(e.carrying) > 0 or reverse == 0:\n",
    "            e.move()\n",
    "\n",
    "        # break in case of infinite loop\n",
    "        if e.movement > 10000: break\n",
    "\n",
    "    # calculate the total cost\n",
    "    total_cost = 0\n",
    "    for _, object in enumerate(b.passengers):\n",
    "        print(object.name, \"going from floor\", object.origin, \"to floor\", object.destination,\"have a cost of\", object.cost)\n",
    "        total_cost += object.cost\n",
    "    print(\"The total cost is\", total_cost)\n",
    "\n",
    "###########################################################################\n",
    "\n",
    "\n",
    "#Strategy 2:\n",
    "# Elevator goes all the way up and then down and opens door at every floor\n",
    "def strategy_2(b, e): #inputs a building and an elevator\n",
    "    awaiting = copy.copy(b.passengers) #list of passengers in the building \n",
    "    e.carrying = [] #reset elevator\n",
    "    while len(awaiting) > 0 or len(e.carrying) > 0: #while there are people to be picked up or people to deliver\n",
    "        print(\"The elevator is at\", e.position, \"carrying\", len(e.carrying), \"passengers\")\n",
    "\n",
    "        # at each floor, first check if there is a passenger to drop\n",
    "        e.drop_passenger()\n",
    "        \n",
    "        # if the elevator still has capacity, check if there is passenger to carry\n",
    "        if len(awaiting) > 0 and len(e.carrying) < e.capacity:\n",
    "            DEL = []\n",
    "            for key, object in enumerate(awaiting): #loop through all waiting passengers in the building\n",
    "                print(e.position, object.name, object.origin)\n",
    "                if e.position == object.origin:\n",
    "                    e.carrying.append(object) #pick up passenger\n",
    "                    DEL.append(key) # add passenger to be removed from elevator calls\n",
    "                    print(\"Picking up\", object.name, \"at floor\", e.position)\n",
    "                    if len(e.carrying) == e.capacity:\n",
    "                        break   # when capacity is reached, stop checking if there are passengers to pick up (break for loop)\n",
    "            if DEL:\n",
    "                awaiting = [p for ind, p in enumerate(awaiting) if ind not in DEL] #remove dropped passengers from awaiting list\n",
    "\n",
    "        # decide if to reverse\n",
    "        if e.position == b.num_of_floor: #if at the top, go down\n",
    "            e.direction = 0\n",
    "        elif e.position == 0: #if on ground floor, go back up\n",
    "            e.direction = 1\n",
    "        \n",
    "        #move elevator in up or down direction\n",
    "        e.move()\n",
    "\n",
    "        # break in case of infinite loop\n",
    "        if e.movement > 100: break\n",
    "\n",
    "    # calculate the total cost\n",
    "    total_cost = 0\n",
    "    for _, object in enumerate(b.passengers):\n",
    "        print(object.name, \"going from floor\", object.origin, \"to floor\", object.destination,\"have a cost of\", object.cost)\n",
    "        total_cost += object.cost\n",
    "    print(\"The total cost is\", total_cost)\n",
    "\n",
    "###########################################################################"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {
    "collapsed": false,
    "scrolled": true
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "The elevator is at 0 carrying 0 passengers\n",
      "The elevator is at 1 carrying 0 passengers\n",
      "Picking up Cznstf at floor 1\n",
      "The elevator is at 2 carrying 1 passengers\n",
      "The elevator is at 3 carrying 1 passengers\n",
      "The elevator is at 4 carrying 1 passengers\n",
      "Picking up Ufauht at floor 4\n",
      "Picking up Husn at floor 4\n",
      "The elevator is at 5 carrying 3 passengers\n",
      "Picking up Tja at floor 5\n",
      "The elevator is at 6 carrying 4 passengers\n",
      "Picking up Dsiu at floor 6\n",
      "The elevator is at 7 carrying 5 passengers\n",
      "The elevator is at 8 carrying 5 passengers\n",
      "Drop off Cznstf at floor 8 with cost 1\n",
      "The elevator is at 9 carrying 4 passengers\n",
      "Picking up Yzdlc at floor 9\n",
      "Picking up Vqm at floor 9\n",
      "The elevator is at 10 carrying 6 passengers\n",
      "The elevator is at 11 carrying 6 passengers\n",
      "Drop off Yzdlc at floor 11 with cost 9\n",
      "The elevator is at 12 carrying 5 passengers\n",
      "The elevator is at 13 carrying 5 passengers\n",
      "The elevator is at 14 carrying 5 passengers\n",
      "The elevator is at 15 carrying 5 passengers\n",
      "Picking up Gkijabzf at floor 15\n",
      "The elevator is at 16 carrying 6 passengers\n",
      "Drop off Dsiu at floor 16 with cost 6\n",
      "The elevator is at 17 carrying 5 passengers\n",
      "Picking up Glf at floor 17\n",
      "The elevator is at 18 carrying 6 passengers\n",
      "Picking up Wfcsmydb at floor 18\n",
      "The elevator is at 19 carrying 7 passengers\n",
      "The elevator is at 20 carrying 7 passengers\n",
      "Drop off Wfcsmydb at floor 20 with cost 18\n",
      "The elevator is at 21 carrying 6 passengers\n",
      "The elevator is at 22 carrying 6 passengers\n",
      "The elevator is at 23 carrying 6 passengers\n",
      "The elevator is at 24 carrying 6 passengers\n",
      "The elevator is at 25 carrying 6 passengers\n",
      "Picking up Rztc at floor 25\n",
      "The elevator is at 26 carrying 7 passengers\n",
      "Drop off Husn at floor 26 with cost 4\n",
      "The elevator is at 27 carrying 6 passengers\n",
      "The elevator is at 28 carrying 6 passengers\n",
      "Drop off Tja at floor 28 with cost 5\n",
      "The elevator is at 29 carrying 5 passengers\n",
      "The elevator is at 30 carrying 5 passengers\n",
      "The elevator is at 31 carrying 5 passengers\n",
      "Drop off Vqm at floor 31 with cost 9\n",
      "The elevator is at 32 carrying 4 passengers\n",
      "The elevator is at 33 carrying 4 passengers\n",
      "The elevator is at 34 carrying 4 passengers\n",
      "The elevator is at 35 carrying 4 passengers\n",
      "The elevator is at 36 carrying 4 passengers\n",
      "The elevator is at 37 carrying 4 passengers\n",
      "Drop off Rztc at floor 37 with cost 25\n",
      "The elevator is at 38 carrying 3 passengers\n",
      "The elevator is at 39 carrying 3 passengers\n",
      "Drop off Glf at floor 39 with cost 17\n",
      "The elevator is at 40 carrying 2 passengers\n",
      "Drop off Gkijabzf at floor 40 with cost 15\n",
      "The elevator is at 41 carrying 1 passengers\n",
      "The elevator is at 42 carrying 1 passengers\n",
      "The elevator is at 43 carrying 1 passengers\n",
      "Drop off Ufauht at floor 43 with cost 4\n",
      "The elevator is at 44 carrying 0 passengers\n",
      "The elevator is at 45 carrying 0 passengers\n",
      "The elevator is at 46 carrying 0 passengers\n",
      "The elevator is at 46 carrying 0 passengers\n",
      "Picking up Hnqvtnix at floor 46\n",
      "The elevator is at 45 carrying 1 passengers\n",
      "Picking up Oxarfv at floor 45\n",
      "The elevator is at 44 carrying 2 passengers\n",
      "Picking up Kfzfuntv at floor 44\n",
      "The elevator is at 43 carrying 3 passengers\n",
      "The elevator is at 42 carrying 3 passengers\n",
      "Picking up Hgstxtm at floor 42\n",
      "The elevator is at 41 carrying 4 passengers\n",
      "Picking up Zfdiyrnb at floor 41\n",
      "Picking up Ybrt at floor 41\n",
      "The elevator is at 40 carrying 6 passengers\n",
      "Drop off Zfdiyrnb at floor 40 with cost 51\n",
      "The elevator is at 39 carrying 5 passengers\n",
      "The elevator is at 38 carrying 5 passengers\n",
      "Picking up Pnrcmqok at floor 38\n",
      "The elevator is at 37 carrying 6 passengers\n",
      "The elevator is at 36 carrying 6 passengers\n",
      "The elevator is at 35 carrying 6 passengers\n",
      "The elevator is at 34 carrying 6 passengers\n",
      "The elevator is at 33 carrying 6 passengers\n",
      "Picking up Kam at floor 33\n",
      "The elevator is at 32 carrying 7 passengers\n",
      "The elevator is at 31 carrying 7 passengers\n",
      "The elevator is at 30 carrying 7 passengers\n",
      "The elevator is at 29 carrying 7 passengers\n",
      "Picking up Qmxkgiem at floor 29\n",
      "The elevator is at 28 carrying 8 passengers\n",
      "Picking up Ycu at floor 28\n",
      "The elevator is at 27 carrying 9 passengers\n",
      "The elevator is at 26 carrying 9 passengers\n",
      "The elevator is at 25 carrying 9 passengers\n",
      "Picking up Mqi at floor 25\n",
      "The elevator is at 24 carrying 10 passengers\n",
      "The elevator is at 23 carrying 10 passengers\n",
      "Drop off Kam at floor 23 with cost 59\n",
      "The elevator is at 22 carrying 9 passengers\n",
      "Picking up Zvgkyy at floor 22\n",
      "The elevator is at 21 carrying 10 passengers\n",
      "The elevator is at 20 carrying 10 passengers\n",
      "The elevator is at 19 carrying 10 passengers\n",
      "Drop off Ybrt at floor 19 with cost 51\n",
      "The elevator is at 18 carrying 9 passengers\n",
      "Drop off Mqi at floor 18 with cost 67\n",
      "The elevator is at 17 carrying 8 passengers\n",
      "Drop off Ycu at floor 17 with cost 64\n",
      "The elevator is at 16 carrying 7 passengers\n",
      "Drop off Zvgkyy at floor 16 with cost 70\n",
      "The elevator is at 15 carrying 6 passengers\n",
      "The elevator is at 14 carrying 6 passengers\n",
      "Drop off Oxarfv at floor 14 with cost 47\n",
      "The elevator is at 13 carrying 5 passengers\n",
      "Drop off Qmxkgiem at floor 13 with cost 63\n",
      "The elevator is at 12 carrying 4 passengers\n",
      "Picking up Leqgcjsu at floor 12\n",
      "The elevator is at 11 carrying 5 passengers\n",
      "Drop off Pnrcmqok at floor 11 with cost 54\n",
      "The elevator is at 10 carrying 4 passengers\n",
      "The elevator is at 9 carrying 4 passengers\n",
      "Drop off Hnqvtnix at floor 9 with cost 46\n",
      "The elevator is at 8 carrying 3 passengers\n",
      "Drop off Kfzfuntv at floor 8 with cost 48\n",
      "The elevator is at 7 carrying 2 passengers\n",
      "The elevator is at 6 carrying 2 passengers\n",
      "The elevator is at 5 carrying 2 passengers\n",
      "The elevator is at 4 carrying 2 passengers\n",
      "The elevator is at 3 carrying 2 passengers\n",
      "Drop off Hgstxtm at floor 3 with cost 50\n",
      "Drop off Leqgcjsu at floor 3 with cost 80\n",
      "The elevator is at 3 carrying 0 passengers\n",
      "The elevator is at 4 carrying 0 passengers\n",
      "The elevator is at 5 carrying 0 passengers\n",
      "The elevator is at 6 carrying 0 passengers\n",
      "The elevator is at 7 carrying 0 passengers\n",
      "The elevator is at 8 carrying 0 passengers\n",
      "The elevator is at 9 carrying 0 passengers\n",
      "The elevator is at 10 carrying 0 passengers\n",
      "The elevator is at 11 carrying 0 passengers\n",
      "The elevator is at 12 carrying 0 passengers\n",
      "The elevator is at 13 carrying 0 passengers\n",
      "The elevator is at 14 carrying 0 passengers\n",
      "The elevator is at 15 carrying 0 passengers\n",
      "The elevator is at 16 carrying 0 passengers\n",
      "The elevator is at 17 carrying 0 passengers\n",
      "The elevator is at 18 carrying 0 passengers\n",
      "The elevator is at 19 carrying 0 passengers\n",
      "The elevator is at 20 carrying 0 passengers\n",
      "The elevator is at 21 carrying 0 passengers\n",
      "The elevator is at 22 carrying 0 passengers\n",
      "The elevator is at 22 carrying 0 passengers\n",
      "Picking up Yufaok at floor 22\n",
      "The elevator is at 21 carrying 1 passengers\n",
      "The elevator is at 20 carrying 1 passengers\n",
      "The elevator is at 19 carrying 1 passengers\n",
      "The elevator is at 18 carrying 1 passengers\n",
      "The elevator is at 17 carrying 1 passengers\n",
      "The elevator is at 16 carrying 1 passengers\n",
      "The elevator is at 15 carrying 1 passengers\n",
      "The elevator is at 14 carrying 1 passengers\n",
      "The elevator is at 13 carrying 1 passengers\n",
      "The elevator is at 12 carrying 1 passengers\n",
      "The elevator is at 11 carrying 1 passengers\n",
      "The elevator is at 10 carrying 1 passengers\n",
      "The elevator is at 9 carrying 1 passengers\n",
      "The elevator is at 8 carrying 1 passengers\n",
      "The elevator is at 7 carrying 1 passengers\n",
      "The elevator is at 6 carrying 1 passengers\n",
      "Drop off Yufaok at floor 6 with cost 108\n",
      "Kfzfuntv going from floor 44 to floor 8 have a cost of 48\n",
      "Qmxkgiem going from floor 29 to floor 13 have a cost of 63\n",
      "Ufauht going from floor 4 to floor 43 have a cost of 4\n",
      "Yzdlc going from floor 9 to floor 11 have a cost of 9\n",
      "Mqi going from floor 25 to floor 18 have a cost of 67\n",
      "Zfdiyrnb going from floor 41 to floor 40 have a cost of 51\n",
      "Kam going from floor 33 to floor 23 have a cost of 59\n",
      "Pnrcmqok going from floor 38 to floor 11 have a cost of 54\n",
      "Rztc going from floor 25 to floor 37 have a cost of 25\n",
      "Cznstf going from floor 1 to floor 8 have a cost of 1\n",
      "Zvgkyy going from floor 22 to floor 16 have a cost of 70\n",
      "Yufaok going from floor 22 to floor 6 have a cost of 108\n",
      "Gkijabzf going from floor 15 to floor 40 have a cost of 15\n",
      "Ycu going from floor 28 to floor 17 have a cost of 64\n",
      "Vqm going from floor 9 to floor 31 have a cost of 9\n",
      "Oxarfv going from floor 45 to floor 14 have a cost of 47\n",
      "Tja going from floor 5 to floor 28 have a cost of 5\n",
      "Wfcsmydb going from floor 18 to floor 20 have a cost of 18\n",
      "Glf going from floor 17 to floor 39 have a cost of 17\n",
      "Leqgcjsu going from floor 12 to floor 3 have a cost of 80\n",
      "Dsiu going from floor 6 to floor 16 have a cost of 6\n",
      "Husn going from floor 4 to floor 26 have a cost of 4\n",
      "Ybrt going from floor 41 to floor 19 have a cost of 51\n",
      "Hgstxtm going from floor 42 to floor 3 have a cost of 50\n",
      "Hnqvtnix going from floor 46 to floor 9 have a cost of 46\n",
      "The total cost is 971\n",
      "The elevator is at 0 carrying 0 passengers\n",
      "0 Kfzfuntv 44\n",
      "0 Qmxkgiem 29\n",
      "0 Ufauht 4\n",
      "0 Yzdlc 9\n",
      "0 Mqi 25\n",
      "0 Zfdiyrnb 41\n",
      "0 Kam 33\n",
      "0 Pnrcmqok 38\n",
      "0 Rztc 25\n",
      "0 Cznstf 1\n",
      "0 Zvgkyy 22\n",
      "0 Yufaok 22\n",
      "0 Gkijabzf 15\n",
      "0 Ycu 28\n",
      "0 Vqm 9\n",
      "0 Oxarfv 45\n",
      "0 Tja 5\n",
      "0 Wfcsmydb 18\n",
      "0 Glf 17\n",
      "0 Leqgcjsu 12\n",
      "0 Dsiu 6\n",
      "0 Husn 4\n",
      "0 Ybrt 41\n",
      "0 Hgstxtm 42\n",
      "0 Hnqvtnix 46\n",
      "The elevator is at 1 carrying 0 passengers\n",
      "1 Kfzfuntv 44\n",
      "1 Qmxkgiem 29\n",
      "1 Ufauht 4\n",
      "1 Yzdlc 9\n",
      "1 Mqi 25\n",
      "1 Zfdiyrnb 41\n",
      "1 Kam 33\n",
      "1 Pnrcmqok 38\n",
      "1 Rztc 25\n",
      "1 Cznstf 1\n",
      "Picking up Cznstf at floor 1\n",
      "1 Zvgkyy 22\n",
      "1 Yufaok 22\n",
      "1 Gkijabzf 15\n",
      "1 Ycu 28\n",
      "1 Vqm 9\n",
      "1 Oxarfv 45\n",
      "1 Tja 5\n",
      "1 Wfcsmydb 18\n",
      "1 Glf 17\n",
      "1 Leqgcjsu 12\n",
      "1 Dsiu 6\n",
      "1 Husn 4\n",
      "1 Ybrt 41\n",
      "1 Hgstxtm 42\n",
      "1 Hnqvtnix 46\n",
      "The elevator is at 2 carrying 1 passengers\n",
      "2 Kfzfuntv 44\n",
      "2 Qmxkgiem 29\n",
      "2 Ufauht 4\n",
      "2 Yzdlc 9\n",
      "2 Mqi 25\n",
      "2 Zfdiyrnb 41\n",
      "2 Kam 33\n",
      "2 Pnrcmqok 38\n",
      "2 Rztc 25\n",
      "2 Zvgkyy 22\n",
      "2 Yufaok 22\n",
      "2 Gkijabzf 15\n",
      "2 Ycu 28\n",
      "2 Vqm 9\n",
      "2 Oxarfv 45\n",
      "2 Tja 5\n",
      "2 Wfcsmydb 18\n",
      "2 Glf 17\n",
      "2 Leqgcjsu 12\n",
      "2 Dsiu 6\n",
      "2 Husn 4\n",
      "2 Ybrt 41\n",
      "2 Hgstxtm 42\n",
      "2 Hnqvtnix 46\n",
      "The elevator is at 3 carrying 1 passengers\n",
      "3 Kfzfuntv 44\n",
      "3 Qmxkgiem 29\n",
      "3 Ufauht 4\n",
      "3 Yzdlc 9\n",
      "3 Mqi 25\n",
      "3 Zfdiyrnb 41\n",
      "3 Kam 33\n",
      "3 Pnrcmqok 38\n",
      "3 Rztc 25\n",
      "3 Zvgkyy 22\n",
      "3 Yufaok 22\n",
      "3 Gkijabzf 15\n",
      "3 Ycu 28\n",
      "3 Vqm 9\n",
      "3 Oxarfv 45\n",
      "3 Tja 5\n",
      "3 Wfcsmydb 18\n",
      "3 Glf 17\n",
      "3 Leqgcjsu 12\n",
      "3 Dsiu 6\n",
      "3 Husn 4\n",
      "3 Ybrt 41\n",
      "3 Hgstxtm 42\n",
      "3 Hnqvtnix 46\n",
      "The elevator is at 4 carrying 1 passengers\n",
      "4 Kfzfuntv 44\n",
      "4 Qmxkgiem 29\n",
      "4 Ufauht 4\n",
      "Picking up Ufauht at floor 4\n",
      "4 Yzdlc 9\n",
      "4 Mqi 25\n",
      "4 Zfdiyrnb 41\n",
      "4 Kam 33\n",
      "4 Pnrcmqok 38\n",
      "4 Rztc 25\n",
      "4 Zvgkyy 22\n",
      "4 Yufaok 22\n",
      "4 Gkijabzf 15\n",
      "4 Ycu 28\n",
      "4 Vqm 9\n",
      "4 Oxarfv 45\n",
      "4 Tja 5\n",
      "4 Wfcsmydb 18\n",
      "4 Glf 17\n",
      "4 Leqgcjsu 12\n",
      "4 Dsiu 6\n",
      "4 Husn 4\n",
      "Picking up Husn at floor 4\n",
      "4 Ybrt 41\n",
      "4 Hgstxtm 42\n",
      "4 Hnqvtnix 46\n",
      "The elevator is at 5 carrying 3 passengers\n",
      "5 Kfzfuntv 44\n",
      "5 Qmxkgiem 29\n",
      "5 Yzdlc 9\n",
      "5 Mqi 25\n",
      "5 Zfdiyrnb 41\n",
      "5 Kam 33\n",
      "5 Pnrcmqok 38\n",
      "5 Rztc 25\n",
      "5 Zvgkyy 22\n",
      "5 Yufaok 22\n",
      "5 Gkijabzf 15\n",
      "5 Ycu 28\n",
      "5 Vqm 9\n",
      "5 Oxarfv 45\n",
      "5 Tja 5\n",
      "Picking up Tja at floor 5\n",
      "5 Wfcsmydb 18\n",
      "5 Glf 17\n",
      "5 Leqgcjsu 12\n",
      "5 Dsiu 6\n",
      "5 Ybrt 41\n",
      "5 Hgstxtm 42\n",
      "5 Hnqvtnix 46\n",
      "The elevator is at 6 carrying 4 passengers\n",
      "6 Kfzfuntv 44\n",
      "6 Qmxkgiem 29\n",
      "6 Yzdlc 9\n",
      "6 Mqi 25\n",
      "6 Zfdiyrnb 41\n",
      "6 Kam 33\n",
      "6 Pnrcmqok 38\n",
      "6 Rztc 25\n",
      "6 Zvgkyy 22\n",
      "6 Yufaok 22\n",
      "6 Gkijabzf 15\n",
      "6 Ycu 28\n",
      "6 Vqm 9\n",
      "6 Oxarfv 45\n",
      "6 Wfcsmydb 18\n",
      "6 Glf 17\n",
      "6 Leqgcjsu 12\n",
      "6 Dsiu 6\n",
      "Picking up Dsiu at floor 6\n",
      "6 Ybrt 41\n",
      "6 Hgstxtm 42\n",
      "6 Hnqvtnix 46\n",
      "The elevator is at 7 carrying 5 passengers\n",
      "7 Kfzfuntv 44\n",
      "7 Qmxkgiem 29\n",
      "7 Yzdlc 9\n",
      "7 Mqi 25\n",
      "7 Zfdiyrnb 41\n",
      "7 Kam 33\n",
      "7 Pnrcmqok 38\n",
      "7 Rztc 25\n",
      "7 Zvgkyy 22\n",
      "7 Yufaok 22\n",
      "7 Gkijabzf 15\n",
      "7 Ycu 28\n",
      "7 Vqm 9\n",
      "7 Oxarfv 45\n",
      "7 Wfcsmydb 18\n",
      "7 Glf 17\n",
      "7 Leqgcjsu 12\n",
      "7 Ybrt 41\n",
      "7 Hgstxtm 42\n",
      "7 Hnqvtnix 46\n",
      "The elevator is at 8 carrying 5 passengers\n",
      "Drop off Cznstf at floor 8 with cost 1\n",
      "8 Kfzfuntv 44\n",
      "8 Qmxkgiem 29\n",
      "8 Yzdlc 9\n",
      "8 Mqi 25\n",
      "8 Zfdiyrnb 41\n",
      "8 Kam 33\n",
      "8 Pnrcmqok 38\n",
      "8 Rztc 25\n",
      "8 Zvgkyy 22\n",
      "8 Yufaok 22\n",
      "8 Gkijabzf 15\n",
      "8 Ycu 28\n",
      "8 Vqm 9\n",
      "8 Oxarfv 45\n",
      "8 Wfcsmydb 18\n",
      "8 Glf 17\n",
      "8 Leqgcjsu 12\n",
      "8 Ybrt 41\n",
      "8 Hgstxtm 42\n",
      "8 Hnqvtnix 46\n",
      "The elevator is at 9 carrying 4 passengers\n",
      "9 Kfzfuntv 44\n",
      "9 Qmxkgiem 29\n",
      "9 Yzdlc 9\n",
      "Picking up Yzdlc at floor 9\n",
      "9 Mqi 25\n",
      "9 Zfdiyrnb 41\n",
      "9 Kam 33\n",
      "9 Pnrcmqok 38\n",
      "9 Rztc 25\n",
      "9 Zvgkyy 22\n",
      "9 Yufaok 22\n",
      "9 Gkijabzf 15\n",
      "9 Ycu 28\n",
      "9 Vqm 9\n",
      "Picking up Vqm at floor 9\n",
      "9 Oxarfv 45\n",
      "9 Wfcsmydb 18\n",
      "9 Glf 17\n",
      "9 Leqgcjsu 12\n",
      "9 Ybrt 41\n",
      "9 Hgstxtm 42\n",
      "9 Hnqvtnix 46\n",
      "The elevator is at 10 carrying 6 passengers\n",
      "10 Kfzfuntv 44\n",
      "10 Qmxkgiem 29\n",
      "10 Mqi 25\n",
      "10 Zfdiyrnb 41\n",
      "10 Kam 33\n",
      "10 Pnrcmqok 38\n",
      "10 Rztc 25\n",
      "10 Zvgkyy 22\n",
      "10 Yufaok 22\n",
      "10 Gkijabzf 15\n",
      "10 Ycu 28\n",
      "10 Oxarfv 45\n",
      "10 Wfcsmydb 18\n",
      "10 Glf 17\n",
      "10 Leqgcjsu 12\n",
      "10 Ybrt 41\n",
      "10 Hgstxtm 42\n",
      "10 Hnqvtnix 46\n",
      "The elevator is at 11 carrying 6 passengers\n",
      "Drop off Yzdlc at floor 11 with cost 9\n",
      "11 Kfzfuntv 44\n",
      "11 Qmxkgiem 29\n",
      "11 Mqi 25\n",
      "11 Zfdiyrnb 41\n",
      "11 Kam 33\n",
      "11 Pnrcmqok 38\n",
      "11 Rztc 25\n",
      "11 Zvgkyy 22\n",
      "11 Yufaok 22\n",
      "11 Gkijabzf 15\n",
      "11 Ycu 28\n",
      "11 Oxarfv 45\n",
      "11 Wfcsmydb 18\n",
      "11 Glf 17\n",
      "11 Leqgcjsu 12\n",
      "11 Ybrt 41\n",
      "11 Hgstxtm 42\n",
      "11 Hnqvtnix 46\n",
      "The elevator is at 12 carrying 5 passengers\n",
      "12 Kfzfuntv 44\n",
      "12 Qmxkgiem 29\n",
      "12 Mqi 25\n",
      "12 Zfdiyrnb 41\n",
      "12 Kam 33\n",
      "12 Pnrcmqok 38\n",
      "12 Rztc 25\n",
      "12 Zvgkyy 22\n",
      "12 Yufaok 22\n",
      "12 Gkijabzf 15\n",
      "12 Ycu 28\n",
      "12 Oxarfv 45\n",
      "12 Wfcsmydb 18\n",
      "12 Glf 17\n",
      "12 Leqgcjsu 12\n",
      "Picking up Leqgcjsu at floor 12\n",
      "12 Ybrt 41\n",
      "12 Hgstxtm 42\n",
      "12 Hnqvtnix 46\n",
      "The elevator is at 13 carrying 6 passengers\n",
      "13 Kfzfuntv 44\n",
      "13 Qmxkgiem 29\n",
      "13 Mqi 25\n",
      "13 Zfdiyrnb 41\n",
      "13 Kam 33\n",
      "13 Pnrcmqok 38\n",
      "13 Rztc 25\n",
      "13 Zvgkyy 22\n",
      "13 Yufaok 22\n",
      "13 Gkijabzf 15\n",
      "13 Ycu 28\n",
      "13 Oxarfv 45\n",
      "13 Wfcsmydb 18\n",
      "13 Glf 17\n",
      "13 Ybrt 41\n",
      "13 Hgstxtm 42\n",
      "13 Hnqvtnix 46\n",
      "The elevator is at 14 carrying 6 passengers\n",
      "14 Kfzfuntv 44\n",
      "14 Qmxkgiem 29\n",
      "14 Mqi 25\n",
      "14 Zfdiyrnb 41\n",
      "14 Kam 33\n",
      "14 Pnrcmqok 38\n",
      "14 Rztc 25\n",
      "14 Zvgkyy 22\n",
      "14 Yufaok 22\n",
      "14 Gkijabzf 15\n",
      "14 Ycu 28\n",
      "14 Oxarfv 45\n",
      "14 Wfcsmydb 18\n",
      "14 Glf 17\n",
      "14 Ybrt 41\n",
      "14 Hgstxtm 42\n",
      "14 Hnqvtnix 46\n",
      "The elevator is at 15 carrying 6 passengers\n",
      "15 Kfzfuntv 44\n",
      "15 Qmxkgiem 29\n",
      "15 Mqi 25\n",
      "15 Zfdiyrnb 41\n",
      "15 Kam 33\n",
      "15 Pnrcmqok 38\n",
      "15 Rztc 25\n",
      "15 Zvgkyy 22\n",
      "15 Yufaok 22\n",
      "15 Gkijabzf 15\n",
      "Picking up Gkijabzf at floor 15\n",
      "15 Ycu 28\n",
      "15 Oxarfv 45\n",
      "15 Wfcsmydb 18\n",
      "15 Glf 17\n",
      "15 Ybrt 41\n",
      "15 Hgstxtm 42\n",
      "15 Hnqvtnix 46\n",
      "The elevator is at 16 carrying 7 passengers\n",
      "Drop off Dsiu at floor 16 with cost 6\n",
      "16 Kfzfuntv 44\n",
      "16 Qmxkgiem 29\n",
      "16 Mqi 25\n",
      "16 Zfdiyrnb 41\n",
      "16 Kam 33\n",
      "16 Pnrcmqok 38\n",
      "16 Rztc 25\n",
      "16 Zvgkyy 22\n",
      "16 Yufaok 22\n",
      "16 Ycu 28\n",
      "16 Oxarfv 45\n",
      "16 Wfcsmydb 18\n",
      "16 Glf 17\n",
      "16 Ybrt 41\n",
      "16 Hgstxtm 42\n",
      "16 Hnqvtnix 46\n",
      "The elevator is at 17 carrying 6 passengers\n",
      "17 Kfzfuntv 44\n",
      "17 Qmxkgiem 29\n",
      "17 Mqi 25\n",
      "17 Zfdiyrnb 41\n",
      "17 Kam 33\n",
      "17 Pnrcmqok 38\n",
      "17 Rztc 25\n",
      "17 Zvgkyy 22\n",
      "17 Yufaok 22\n",
      "17 Ycu 28\n",
      "17 Oxarfv 45\n",
      "17 Wfcsmydb 18\n",
      "17 Glf 17\n",
      "Picking up Glf at floor 17\n",
      "17 Ybrt 41\n",
      "17 Hgstxtm 42\n",
      "17 Hnqvtnix 46\n",
      "The elevator is at 18 carrying 7 passengers\n",
      "18 Kfzfuntv 44\n",
      "18 Qmxkgiem 29\n",
      "18 Mqi 25\n",
      "18 Zfdiyrnb 41\n",
      "18 Kam 33\n",
      "18 Pnrcmqok 38\n",
      "18 Rztc 25\n",
      "18 Zvgkyy 22\n",
      "18 Yufaok 22\n",
      "18 Ycu 28\n",
      "18 Oxarfv 45\n",
      "18 Wfcsmydb 18\n",
      "Picking up Wfcsmydb at floor 18\n",
      "18 Ybrt 41\n",
      "18 Hgstxtm 42\n",
      "18 Hnqvtnix 46\n",
      "The elevator is at 19 carrying 8 passengers\n",
      "19 Kfzfuntv 44\n",
      "19 Qmxkgiem 29\n",
      "19 Mqi 25\n",
      "19 Zfdiyrnb 41\n",
      "19 Kam 33\n",
      "19 Pnrcmqok 38\n",
      "19 Rztc 25\n",
      "19 Zvgkyy 22\n",
      "19 Yufaok 22\n",
      "19 Ycu 28\n",
      "19 Oxarfv 45\n",
      "19 Ybrt 41\n",
      "19 Hgstxtm 42\n",
      "19 Hnqvtnix 46\n",
      "The elevator is at 20 carrying 8 passengers\n",
      "Drop off Wfcsmydb at floor 20 with cost 18\n",
      "20 Kfzfuntv 44\n",
      "20 Qmxkgiem 29\n",
      "20 Mqi 25\n",
      "20 Zfdiyrnb 41\n",
      "20 Kam 33\n",
      "20 Pnrcmqok 38\n",
      "20 Rztc 25\n",
      "20 Zvgkyy 22\n",
      "20 Yufaok 22\n",
      "20 Ycu 28\n",
      "20 Oxarfv 45\n",
      "20 Ybrt 41\n",
      "20 Hgstxtm 42\n",
      "20 Hnqvtnix 46\n",
      "The elevator is at 21 carrying 7 passengers\n",
      "21 Kfzfuntv 44\n",
      "21 Qmxkgiem 29\n",
      "21 Mqi 25\n",
      "21 Zfdiyrnb 41\n",
      "21 Kam 33\n",
      "21 Pnrcmqok 38\n",
      "21 Rztc 25\n",
      "21 Zvgkyy 22\n",
      "21 Yufaok 22\n",
      "21 Ycu 28\n",
      "21 Oxarfv 45\n",
      "21 Ybrt 41\n",
      "21 Hgstxtm 42\n",
      "21 Hnqvtnix 46\n",
      "The elevator is at 22 carrying 7 passengers\n",
      "22 Kfzfuntv 44\n",
      "22 Qmxkgiem 29\n",
      "22 Mqi 25\n",
      "22 Zfdiyrnb 41\n",
      "22 Kam 33\n",
      "22 Pnrcmqok 38\n",
      "22 Rztc 25\n",
      "22 Zvgkyy 22\n",
      "Picking up Zvgkyy at floor 22\n",
      "22 Yufaok 22\n",
      "Picking up Yufaok at floor 22\n",
      "22 Ycu 28\n",
      "22 Oxarfv 45\n",
      "22 Ybrt 41\n",
      "22 Hgstxtm 42\n",
      "22 Hnqvtnix 46\n",
      "The elevator is at 23 carrying 9 passengers\n",
      "23 Kfzfuntv 44\n",
      "23 Qmxkgiem 29\n",
      "23 Mqi 25\n",
      "23 Zfdiyrnb 41\n",
      "23 Kam 33\n",
      "23 Pnrcmqok 38\n",
      "23 Rztc 25\n",
      "23 Ycu 28\n",
      "23 Oxarfv 45\n",
      "23 Ybrt 41\n",
      "23 Hgstxtm 42\n",
      "23 Hnqvtnix 46\n",
      "The elevator is at 24 carrying 9 passengers\n",
      "24 Kfzfuntv 44\n",
      "24 Qmxkgiem 29\n",
      "24 Mqi 25\n",
      "24 Zfdiyrnb 41\n",
      "24 Kam 33\n",
      "24 Pnrcmqok 38\n",
      "24 Rztc 25\n",
      "24 Ycu 28\n",
      "24 Oxarfv 45\n",
      "24 Ybrt 41\n",
      "24 Hgstxtm 42\n",
      "24 Hnqvtnix 46\n",
      "The elevator is at 25 carrying 9 passengers\n",
      "25 Kfzfuntv 44\n",
      "25 Qmxkgiem 29\n",
      "25 Mqi 25\n",
      "Picking up Mqi at floor 25\n",
      "The elevator is at 26 carrying 10 passengers\n",
      "Drop off Husn at floor 26 with cost 4\n",
      "26 Kfzfuntv 44\n",
      "26 Qmxkgiem 29\n",
      "26 Zfdiyrnb 41\n",
      "26 Kam 33\n",
      "26 Pnrcmqok 38\n",
      "26 Rztc 25\n",
      "26 Ycu 28\n",
      "26 Oxarfv 45\n",
      "26 Ybrt 41\n",
      "26 Hgstxtm 42\n",
      "26 Hnqvtnix 46\n",
      "The elevator is at 27 carrying 9 passengers\n",
      "27 Kfzfuntv 44\n",
      "27 Qmxkgiem 29\n",
      "27 Zfdiyrnb 41\n",
      "27 Kam 33\n",
      "27 Pnrcmqok 38\n",
      "27 Rztc 25\n",
      "27 Ycu 28\n",
      "27 Oxarfv 45\n",
      "27 Ybrt 41\n",
      "27 Hgstxtm 42\n",
      "27 Hnqvtnix 46\n",
      "The elevator is at 28 carrying 9 passengers\n",
      "Drop off Tja at floor 28 with cost 5\n",
      "28 Kfzfuntv 44\n",
      "28 Qmxkgiem 29\n",
      "28 Zfdiyrnb 41\n",
      "28 Kam 33\n",
      "28 Pnrcmqok 38\n",
      "28 Rztc 25\n",
      "28 Ycu 28\n",
      "Picking up Ycu at floor 28\n",
      "28 Oxarfv 45\n",
      "28 Ybrt 41\n",
      "28 Hgstxtm 42\n",
      "28 Hnqvtnix 46\n",
      "The elevator is at 29 carrying 9 passengers\n",
      "29 Kfzfuntv 44\n",
      "29 Qmxkgiem 29\n",
      "Picking up Qmxkgiem at floor 29\n",
      "The elevator is at 30 carrying 10 passengers\n",
      "The elevator is at 31 carrying 10 passengers\n",
      "Drop off Vqm at floor 31 with cost 9\n",
      "31 Kfzfuntv 44\n",
      "31 Zfdiyrnb 41\n",
      "31 Kam 33\n",
      "31 Pnrcmqok 38\n",
      "31 Rztc 25\n",
      "31 Oxarfv 45\n",
      "31 Ybrt 41\n",
      "31 Hgstxtm 42\n",
      "31 Hnqvtnix 46\n",
      "The elevator is at 32 carrying 9 passengers\n",
      "32 Kfzfuntv 44\n",
      "32 Zfdiyrnb 41\n",
      "32 Kam 33\n",
      "32 Pnrcmqok 38\n",
      "32 Rztc 25\n",
      "32 Oxarfv 45\n",
      "32 Ybrt 41\n",
      "32 Hgstxtm 42\n",
      "32 Hnqvtnix 46\n",
      "The elevator is at 33 carrying 9 passengers\n",
      "33 Kfzfuntv 44\n",
      "33 Zfdiyrnb 41\n",
      "33 Kam 33\n",
      "Picking up Kam at floor 33\n",
      "The elevator is at 34 carrying 10 passengers\n",
      "The elevator is at 35 carrying 10 passengers\n",
      "The elevator is at 36 carrying 10 passengers\n",
      "The elevator is at 37 carrying 10 passengers\n",
      "The elevator is at 38 carrying 10 passengers\n",
      "The elevator is at 39 carrying 10 passengers\n",
      "Drop off Glf at floor 39 with cost 17\n",
      "39 Kfzfuntv 44\n",
      "39 Zfdiyrnb 41\n",
      "39 Pnrcmqok 38\n",
      "39 Rztc 25\n",
      "39 Oxarfv 45\n",
      "39 Ybrt 41\n",
      "39 Hgstxtm 42\n",
      "39 Hnqvtnix 46\n",
      "The elevator is at 40 carrying 9 passengers\n",
      "Drop off Gkijabzf at floor 40 with cost 15\n",
      "40 Kfzfuntv 44\n",
      "40 Zfdiyrnb 41\n",
      "40 Pnrcmqok 38\n",
      "40 Rztc 25\n",
      "40 Oxarfv 45\n",
      "40 Ybrt 41\n",
      "40 Hgstxtm "
     ]
    },
    {
     "name": "stderr",
     "output_type": "stream",
     "text": [
      "WARNING: Some output was deleted.\n"
     ]
    }
   ],
   "source": [
    "building_1 = Building(50, 25)\n",
    "elevator_1 = Elevator(10)\n",
    "\n",
    "strategy_1(copy.copy(building_1), copy.copy(elevator_1))\n",
    "strategy_2(copy.copy(building_1), copy.copy(elevator_1))"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (Anaconda)",
   "language": "python",
   "name": "anaconda3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.5.4"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 0
}